#! /usr/bin/env python3
#conddef.py.py

#Copyright (c) 2018, David Anderson
#All rights reserved.
#
#Redistribution and use in source and binary forms, with
#or without modification, are permitted provided that the
#following conditions are met:
#
#    Redistributions of source code must retain the above
#    copyright notice, this list of conditions and the following
#    disclaimer.
#
#    Redistributions in binary form must reproduce the above
#    copyright notice, this list of conditions and the following
#    disclaimer in the documentation and/or other materials
#    provided with the distribution.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
#CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
#INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
#OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
#CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
#NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
#HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
#CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
#OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
#EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#Reads  
  #define x y
#lines
#and turns them into
  #ifndef x
  #define x y
  #endif

import os
import sys

def getfirstthree(s):
  out = ""
  if len(s) >= 3:
     out = "".join((s[0],s[1],s[2]))
  return out

def reader(path):
  try:
    file = open(path,"r")
  except IOError as  message:
    print("File could not be opened: ", message,file=sys.stderr)
    sys.exit(1)
  out = []
  iline = 0
  saveword = ""
  for line in file:
    iline = int(iline) + 1
    l2 = line.rstrip()
    wds = l2.split()
    if len(wds) < 3:
       continue
    if wds[0] != "#define":
      continue
    mdef = wds[1]
    val = wds[2]
    out+= [(mdef,val)] 
  file.close()
  return out
if __name__ == '__main__':
  filename = ""
  if len(sys.argv) == 3:
    if sys.argv[1] == "--infile":
      filename = sys.argv[2]
    else:
      print("argument ", sys.argv[1]," ignored")
  else:
    print("Usage: conddef.py --infile <filepath>")
    sys.exit(1)

  full_list = reader(filename)
  maxlen = 16
  for f in full_list:
    (mdef,val) = f
    dlen = len(mdef)
    if int(dlen) > int(maxlen):
       maxlen = int(dlen)
  pstr = "#define %-" + str(maxlen) + "s %s"
  initial = ""
  for f in full_list:
     (mdef,val) = f
     ourthree = getfirstthree(mdef)
     if initial != ourthree:
        print("")
        initial = ourthree
     print("#ifndef",mdef)
     print(pstr % (mdef,val))
     print("#endif")
    



#!/usr/bin/env python3
#   Ensure for every incude <stdlib.h> that we
#    have the ifdef HAVE_STDLIB_H  and endif
#    bracketing it.
#    And so on.

import os
import sys

includelist = [
("HAVE_ALLOCA_H","alloca.h"), 
("HAVE_DLFCN_H","dlfcn.h"), 
("HAVE_ELFACCESS_H","elfaccess.h"), 
("HAVE_ELF_H","elf.h"),  
("HAVE_INTTYPES_H","inttypes.h"),
("HAVE_LIBELF_H","libelf.h"), 
("HAVE_LIBELF_LIBELF_H","libelf/libelf.h"), 
("HAVE_MALLOC_H","malloc.h"), 
("HAVE_MEMORY_H","memory.h"), 
("HAVE_REGEX_H","regex.h"), 
("HAVE_SGIDEFS_H","sgidefs.h"),
("HAVE_STDINT_H","stdint.h"),
("HAVE_STDLIB_H","stdlib.h"), 
("HAVE_STRINGS_H","strings.h"), 
("HAVE_STRING_H","string.h"),
("HAVE_SYS_ELF_386_H","sys/elf_386.h"), 
("HAVE_SYS_ELF_AMD64_H","sys/elf/amd64_h"),
("HAVE_SYS_ELF_SPARC_H","sys/elf_sparc_h"),
("HAVE_SYS_IA64_ELF_H","sys/ia64_elf.h"),
("HAVE_SYS_STAT_H","sys/stat.h"), 
("HAVE_SYS_TYPES_H","sys/types.h"),
("HAVE_UNISTD_H","unistd.h"), 
("HAVE_WINDOWS_H","windows.h"), 
("HAVE_ZLIB_H","zlib.h"), 
]

havedict = {}
incldict = {}
for x in includelist:
  (h,i) = x
  havedict[h] = i
  incldict[i] = h

ifdef = "#ifdef"
endif = "#endif"
include = "#include"

def lineisofinterest(line):
  if line.startswith(ifdef):
    return ifdef
  if line.startswith(include):
    return include
  if line.startswith(endif):
    return endif
  return False

def isolatetag(t,iline):
  if t[0] == '<':
    x = t.replace('<','')
    y = x.replace('>','')
    return y
  elif t[0] == '"':
    x = t.replace('"','')
    return x
  else:
    print("Something very wrong with include on line ",iline)
    sys.exit(1)

def scanfile(path):
  try:
    fx = open(path,"r")
  except IOError as  message:
    print("File",path,"could not be opened: ", message)
    sys.exit(1)

  ifdefline = 0
  insideifdef = False
  ifdefdepth = 0
  ifdeftag = ''
  includetag = ''
  includeline = 0
  endifline = 0
  iline = 0
  while 1:
    iline = int(iline)+1
    try:
      rline = fx.readline()
    except IOError as message:
      print("read fails, line ",iline,message)
      sys.exit(1)
    if rline == '':
      break
    tline = rline.strip()
    x = lineisofinterest(tline)
    if not x:
      continue
    wds = tline.split()
    if x == ifdef:
      if ifdefdepth > 0:
        print("nested include of ",wds[1],"line",iline)
      else:
        ifdeftag = wds[1]
      ifdefdepth = int(ifdefdepth) +1
      #print("ifdef depth",ifdefdepth, " at ",wds[1], "line",iline)
    elif x == include:
      includetag=isolatetag(wds[1],iline) 
      if ifdefdepth == 0:
        if includetag in incldict:
            print("Found include ",includetag,"not in ifdef/endif")
      elif ifdefdepth > 1:
        pass
      else:
        if ifdeftag in havedict:
           if includetag in incldict:
              #normal
              pass
           else:
              if ifdefdepth == 0:
                print("Found include ",includetag,"not in ifdef/endif")
              elif ifdefdepth == 1:
                print("Found have tag ",ifdeftag,"with unk include ",\
                includetag)
              else:
                pass
        else:
           if includetag in incldict:
              print("Found unk have tag ",ifdeftag,"with include ",\
              includetag);
    elif x == endif:
      if ifdefdepth > 1:
        print("end nested include line ",iline);
      else:
        pass
      ifdefdepth = int(ifdefdepth) -1
      #print("ifdef depth",ifdefdepth, " at ",wds[1],"line",iline)
    else:
       print("Fail at line ",iline);
       sys.exit(1)
  if ifdefdepth > 0:  
    print("ends improperly, not nested ifdef ")


if __name__ == '__main__':
  i = 1
  ct = 0
  while i < len(sys.argv):
     path = sys.argv[i]
     print("=======path=",path)
     scanfile(path);
     ct = int(ct)+1
     i = int(i)+1
  if int(ct) < 1:
     print("file name argument(s) required");
     sys.exit(1)





#!/usr/bin/python3
#  Copyright (c) 2016-2016 David Anderson.
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#  * Neither the name of the example nor the
#    names of its contributors may be used to endorse or promote products
#    derived from this software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY David Anderson ''AS IS'' AND ANY
#  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
#  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
#  DISCLAIMED. IN NO EVENT SHALL David Anderson BE LIABLE FOR ANY
#  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
#  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
#  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
#  OF SUCH DAMAGE.


import sys

# Use only <pre> or </pre> all by itself in data.xml.
# No other data on either of such lines.
# All the lines between these two markers should be
# shown in individual lines.
def xmlize(linea,inhtml,inpre):
  outi =  []
  l = linea
  if l.find("<pre>") != -1:
     if inhtml == 'y':
       s2 = '</p>' +l + '\n'
     else:
       s2 = l + '\n'
     inpre = 'y'
     return s2,inpre
  if l.find("</pre>") != -1:
     if inhtml == 'y':
       s2 = l + '\n' + "<p>"
     else:
       s2 = l + '\n' 
     inpre = 'n'
     return s2, inpre
  if inpre == 'y' and inhtml == 'n':
    outi += ["<preline>"]
  for c in l:
    if c == '<':
       outi += ["&lt;"]
    elif c == '>':
       outi += ["&gt;"]
    elif c == "&":
       outi += ["&amp;"]
    #elif c == "'":
    #   outi += ["&apos;"]
    elif c == '"':
       outi += ["&quot;"]
    else:
       outi += [c]
  if inpre == 'y' and inhtml == 'n':
    outi += ["</preline>"]
  outi += ["\n"]
  s2 = ''.join(outi)
  return s2,inpre

def paraline(name,linea):
  inpre = 'n'
  out = ''
  if len(linea) <1:
    out = "<p>" + name + ":"+ "</p>"
    return out
  out = "<p>" + name + ": "
  out +=linea 
  out += "</p>"
  return out;

def paralines(name,lines):
  inpre = 'n'
  if len(lines) <1:
    out = "<p>" + name + ":"+ "</p>"
    return out
  out = "<p>" + name + ": "
  for lin in lines:
     f,inpre = xmlize(lin,'y',inpre)
     out += f
  out += "</p>"
  return out;

def para(name,str):
  if str ==  None:
      out = "<p>" + name + ":"+ "</p>"
  elif len(str) > 0:
      out = "<p>" + name + ":&nbsp" + str + "</p>"
  else:
      out = "<p>" + name + ":"+ "</p>"
  return out


class bugrecord:

  def __init__(self,dwid):
    self._id= dwid.strip()
    self._cve  = ''
    self._datereported  = ''
    self._reportedby  = ''
    self._vulnerability  = []
    self._product  = ''
    self._description  = []
    self._datefixed  = ''
    self._references  = []
    self._gitfixid  = ''
    self._tarrelease  = ''

  def setcve(self,pubid):
    if self._cve != '':
      print("Duplicate cve ",self._cve,pubid)
      sys.exit(1)
    self._cve  = pubid.strip()
  def setdatereported(self,rep):
    if self._datereported != '':
      print("Duplicate datereported ",self._datereported,rep)
      sys.exit(1)
    self._datereported  = rep.strip()
  def setreportedby(self,rep):
    if self._reportedby != '':
      print("Duplicate reportedby ",self._reportedby,rep)
      sys.exit(1)
    self._reportedby  = rep.strip()
  def setvulnerability(self,vuln):
    if len(self._vulnerability) != 0:
      print("Duplicate vulnerability ",self._vulnerability,vuln)
      sys.exit(1)
    self._vulnerability  = vuln
  def setproduct(self,p):
    if len(self._product) != 0:
      print("Duplicate product ",self._product,p)
      sys.exit(1)
    self._product  = p.strip()
  def setdescription(self,d):
    if len(self._description) != 0:
      print("Duplicate description ",self._description,d)
      sys.exit(1)
    self._description  = d
  def setdatefixed(self,d):
    if len(self._datefixed) != 0:
      print("Duplicate datefixed ",self._datefixed,d)
      sys.exit(1)
    self._datefixed  = d.strip()
  def setreferences(self,r):
    if len(self._references) != 0:
      print("Duplicate references ",self._references,r)
      sys.exit(1)
    self._references  = r
  def setgitfixid(self,g):
    if len(self._gitfixid) != 0:
      print("Duplicate gitfixid ",self._gitfixid,g)
      sys.exit(1)
    self._gitfixid  = g.strip()
  def settarrelease(self,g):
    if len(self._tarrelease) != 0:
      print("Duplicate tarrelease ",self._tarrelease,g)
      sys.exit(1)
    self._tarrelease  = g.strip()
  def plist(self,title,lines):
    if lines == None:
      print(title)
      return
    if len(lines) == 1:
      print(title,lines[0])
      return
    print(title)
    for l in lines:
        print(l)

  def printbug(self):
    print("")
    print("id:",self._id)
    print("cve:",self._cve)
    print("datereported:",self._datereported)
    print("reportedby:",self._reportedby)
    self.plist("vulnerability:",self._vulnerability)
    print("product:",self._product)
    self.plist("description:",self._description)
    print("datefixed:",self._datefixed)
    self.plist("references:",self._references)
    print("gitfixid:",self._gitfixid)
    print("tarrelease:",self._tarrelease)

  def generate_html(self):
    s5= ''.join(self._id)
    t = ''.join(['<h3 id="',s5,'">',self._id,'</h3>'])
    txt = [t]

    inpre = 'n'
    s,inp= xmlize(self._id,'y',inpre)
    t = paraline("id",s)
    txt += [t]
    s,inp= xmlize(self._cve,'y',inpre)
    t = paraline("cve",s)
    txt += [t]

    s,inp= xmlize(self._datereported,'y',inpre)
    t = paraline("datereported",s)
    txt += [t]

    s,inp= xmlize(self._reportedby,'y',inpre)
    t = paraline("reportedby",s)
    txt += [t]
 
    #MULTI
    t = paralines("vulnerability",self._vulnerability)
    txt += [t]


    s,inp= xmlize(self._product,'y',inpre)
    t = paraline("product",s)
    txt += [t]
    
    #MULTI
    t = paralines("description",self._description)
    txt += [t]

    s,inp= xmlize(self._datefixed,'y',inpre)
    t = paraline("datefixed",s)
    txt += [t]

    #MULTI
    t = paralines("references",self._references)
    txt += [t]

    s,inp= xmlize(self._gitfixid,'y',inpre)
    t = paraline("gitfixid",s)
    txt += [t]

    s,inp= xmlize(self._tarrelease,'y',inpre)
    t = paraline("tarrelease",s)
    txt += [t]

    t = '<p> <a href="#top">[top]</a> </p>'
    txt += [t]
    return txt

  def paraxml(self,start,main,term):
    # For single line xml remove the newline from the main text line.
    out = start
    l=main.strip()
    if len(l) > 0:
      out += l 
    out += term + "\n" 
    return out
  def paraxmlN(self,start,main,term):
    # For multi line xml leave newlines present.
    out = start 
    inpre = 'n'
    for x in main:
      l=x.rstrip()
      t,inpre = xmlize(l,'n',inpre);
      if len(t) > 0:
        out += t
    out += term + "\n"
    return out


  def generate_xml(self):
    txt=[]
    t = '<dwbug>'
    txt += [t]
     
    inpre = 'n'
    s,inpre= xmlize(self._id,'n',inpre)
    s = self.paraxml('<dwid>',s,'</dwid>')

    s,inpre= xmlize(self._cve,'n',inpre)
    t = self.paraxml('<cve>',s,'</cve>')
    txt += [t]

    s,inpre= xmlize(self._datereported,'n',inpre)
    t = self.paraxml('<datereported>',s,'</datereported>')
    txt += [t];

    s,inpre= xmlize(self._reportedby,'n',inpre)
    t = self.paraxml('<reportedby>',s,'</reportedby>')
    txt += [t];

    s,inpre= xmlize(self._product,'n',inpre)
    t = self.paraxml('<product>',s,'</product>')
    txt += [t];

    
    #MULTI
    p = self._vulnerability
    t = self.paraxmlN("<vulnerability>",p,"</vulnerability>")
    txt += [t]


    #MULTI
    p = self._description
    t = self.paraxmlN("<description>",p,"</description>")
    txt += [t]


    s,inpre= xmlize(self._datefixed,'n',inpre)
    t = self.paraxml('<datefixed>',s,'</datefixed>')
    txt += [t];

    #MULTI
    p = self._references    
    t = self.paraxmlN("<references>",p,"</references>")
    txt += [t]

    s,inpre= xmlize(self._gitfixid,'n',inpre)
    t = self.paraxml('<gitfixid>',s,'</gitfixid>')
    txt += [t];

    s,inpre= xmlize(self._tarrelease,'n',inpre)
    t = self.paraxml('<tarrelease>',s,'</tarrelease>')
    txt += [t];

    t = '</dwbug>'
    txt += [t];
    return txt



#!/usr/bin/python3
#  Copyright (c) 2016-2016 David Anderson.
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#  * Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#  * Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#  * Neither the name of the example nor the
#    names of its contributors may be used to endorse or promote products
#    derived from this software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY David Anderson ''AS IS'' AND ANY
#  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
#  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
#  DISCLAIMED. IN NO EVENT SHALL David Anderson BE LIABLE FOR ANY
#  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
#  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
#  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
#  OF SUCH DAMAGE.

import os
import sys
sys.path.append(os.path.abspath("/home/davea/dwarf/code/bugxml"))
import bugrecord

def ignore_this_line(d,inrecord):
  if len(d) < 1:
    if inrecord == "y":
      return "n"
    else:
      return "y"
  s = str(d)
  if s[0] == '#':
    return "y"
  return "n"

def closeouttext(bugrec,intext,text,linecount):
  if intext == 'd':
    bugrec.setdescription(text)
    return
  elif intext == 'v':
    bugrec.setvulnerability(text)
    return
  elif intext == 'r':
    bugrec.setreferences(text)
    return
  if intext == "":
    return
  print("bogus closeout line at line ",linecount)
  sys.exit(1)


def readbugs(iname):
  name = iname
  if len(name) == 0:
    name = "/home/davea/dwarf/code/bugxml/data.txt"
  try:
      file = open(name,"r")
  except IOError as message:
      print("failed to open ",name, message)

  inrecord = "n"
  linecount = 0
  text = []
  usedid ={}
  intext = ''
  bugrec = ''
  buglist = []
  while 1:
    try:
      rec = file.readline()
    except EOFError:
        break
    if len(rec) < 1:
      # eof
      break
    linecount += 1
    if ignore_this_line(rec,inrecord) == "y":
      continue
    rec = rec.rstrip()
    if inrecord == "n":
      if len(rec) == 0:
        continue
      if  rec.find(":") == -1:
        print("bogus non-blank line at line ",linecount)
        sys.exit(1)
    if inrecord == "y" and len(rec) > 0:
      # A multi line entry may have ":" in it.
      if intext != "" and rec[0] == ' ':
        s3 = ''.join(rec)
        text += [s3]
        continue
    low = rec.find(":")
    fldname = rec[0:low+1]
    fldval = rec[low+1:]
    if fldname == "id:": 
      if inrecord == "y":
        print("bogus id: at line ",linecount)
        sys.exit(1)
      inrecord = "y"
      f = fldval.strip()
      if f in usedid:
        print("Duplicate Key:",f,"Giving up.")
        sys.exit(1)
      usedid[f] = 1
      s4= ''.join(fldval)
      bugrec = bugrecord.bugrecord(s4)
    elif fldname == "cve:":
      closeouttext(bugrec,intext,text,linecount),
      intext = ""
      text = []
      s4= ''.join(fldval)
      bugrec.setcve(s4)
    elif fldname == "datereported:":
      closeouttext(bugrec,intext,text,linecount),
      intext = ""
      text = []
      s4= ''.join(fldval)
      bugrec.setdatereported(s4)
    elif fldname == "reportedby:":
      closeouttext(bugrec,intext,text,linecount),
      intext = ""
      text = []
      s4= ''.join(fldval)
      bugrec.setreportedby(s4)
    elif fldname == "vulnerability:":
      closeouttext(bugrec,intext,text,linecount),
      intext = 'v'
      text = []
      if len(fldval) > 0:
        s4= ''.join(fldval)
        text = [s4]

    elif fldname == "product:":
      closeouttext(bugrec,intext,text,linecount),
      intext = ""
      text = []
      s4= ''.join(fldval)
      bugrec.setproduct(s4)
    elif fldname == "description:":
      closeouttext(bugrec,intext,text,linecount),
      text = []
      intext = 'd'
      if len(fldval) > 0:
        s4= ''.join(fldval)
        text = [s4]

    elif fldname == "datefixed:":
      closeouttext(bugrec,intext,text,linecount),
      text = []
      intext = ""
      s4= ''.join(fldval)
      bugrec.setdatefixed(s4)
    elif fldname == "references:":
      closeouttext(bugrec,intext,text,linecount),
      text = []
      intext = 'r'
      if len(fldval) > 0:
        s4= ''.join(fldval)
        text = [s4]
    elif fldname == "gitfixid:":
      closeouttext(bugrec,intext,text,linecount),
      text = []
      intext = ""
      s4= ''.join(fldval)
      bugrec.setgitfixid(s4)
    elif fldname == "tarrelease:":
      closeouttext(bugrec,intext,text,linecount),
      text = []
      intext = ""
      s4= ''.join(fldval)
      bugrec.settarrelease(s4)
    elif fldname == "endrec:":
      closeouttext(bugrec,intext,text,linecount),
      text = []
      if inrecord == "n":
         print("bogus endrec: at line ",linecount)
         sys.exit(1)
      buglist += [bugrec]
      inrecord = "n"
      text = []
      intext = ""
      inrecord = "n"
  file.close()
  return buglist







def sort_by_id(myl):
  """ Sort the list of objects by name. """
  auxiliary = [ ( x._id, x) for x in myl ]
  auxiliary.sort()
  return [ x[1] for x in auxiliary ]


def write_line(file,l):
  file.write(l + "\n")

def write_all_lines(file,txt):
  for t in txt:
     write_line(file,t)

def generatehtml(list2,name):
  try:
    file = open(name,"w")
  except IOError as message:
    print("failed to open ",name, message)
    sys.exit(1)
  for b in list2:
    txt=b.generate_html()
    write_all_lines(file,txt)
  write_line(file,"</body>")
  write_line(file,"</html>")
  file.close()

def generatexml(list2,name):
  try:
    file = open(name,"w")
  except IOError as message:
    print("failed to open ",name, message)
    sys.exit(1)
  t = '<?xml version="1.0" encoding="us-ascii"?>'
  write_line(file,t)
  write_line(file,"<dwarfbug>")
  for b in list2:
    txt=b.generate_xml()
    write_all_lines(file,txt)
  write_line(file,"</dwarfbug>")
  file.close()




if __name__ == '__main__':
  list = readbugs("")
  list2  = sort_by_id(list)
  list2.reverse()
  #for b in list2:
  #  b.printbug()

  generatehtml(list2,"./dwarfbugtail")
  generatexml(list2,"./dwarfbuglohi.xml")

  list2.reverse()
  generatehtml(list2,"./dwarfbuglohitail")




#! /usr/bin/env python3
#funcfindersrcs.py

#Copyright (c) 2018, David Anderson
#All rights reserved.
#
#Redistribution and use in source and binary forms, with
#or without modification, are permitted provided that the
#following conditions are met:
#
#    Redistributions of source code must retain the above
#    copyright notice, this list of conditions and the following
#    disclaimer.
#
#    Redistributions in binary form must reproduce the above
#    copyright notice, this list of conditions and the following
#    disclaimer in the documentation and/or other materials
#    provided with the distribution.
#
#THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
#CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
#INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
#OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
#CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
#NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
#HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
#CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
#OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
#EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import os
import sys

#Reads the producer code source and produces a concise list
#of the implemented functions, sorted by function name.

# Run as
# funcfindersrcs.py 
# or
# funcfindersrcs.py --nonumbers


filelist =  [
"pro_alloc.c",
"pro_alloc.c",
"pro_arange.c",
"pro_die.c",
"pro_dnames.c",
"pro_encode_nm.c",
"pro_error.c",
"pro_expr.c",
"pro_finish.c",
"pro_forms.c",
"pro_frame.c",
"pro_funcs.c",
"pro_init.c",
"pro_line.c",
"pro_macinfo.c",
"pro_pubnames.c",
"pro_reloc.c",
"pro_reloc_stream.c",
"pro_reloc_symbolic.c",
"pro_section.c",
"pro_types.c",
"pro_vars.c",
"pro_weaks.c",
"dwarf_util.c",
"dwarf_util.c",
]


def looks_like_func(s):
  fwds = s.split("(")
  if len(fwds) < 2:
    #print("Not func",fwds)
    return "n","","",""
  fwds2 = fwds[0].split(" ")
  if len(fwds2) < 2:
    #print("Not func",fwds)
    return "n","","",""
  if len(fwds2) == 2:
    return "y",fwds2[1],fwds2[0],""
  if len(fwds2) == 3:
    return "y",fwds2[2],fwds2[0],fwds2[1]
  return "n","","",""

def reader(path):
  try:
    file = open(path,"r")
  except IOError as  message:
    print("File could not be opened: ", message,file=sys.stderr)
    sys.exit(1)
  out = []
  iline = 0
  prevline = ''
  all_the_lines = file.read().splitlines()
  file.close()
  #print("No. Lines:",len(all_the_lines))
  for line in all_the_lines:
    l2 = line.rstrip()
    iline = int(iline) + 1
    if l2.startswith(" ") == 1:
      continue
    if l2.startswith("/") == 1:
      continue
    if l2.startswith("dwarf_") == 1:
      l3 = l2.split("(");
      if len(l3) < 2:
        printf(" CHECKME: odd-a ",l3);
        continue
      if l3[1].startswith(")") == 1:
        continue
      out += [(l3[0],prev,iline,path)] 
    else:
       prev=l2.strip()
  return out
if __name__ == '__main__':
  showlinenum = "y"
  if len(sys.argv) > 1:
    if sys.argv[1] == "--nonumbers":
      showlinenum = "n"
    else:
      print("argument ", sys.argv[1],"ignored") 
  funcl = []
  for path in filelist:
    funcl += reader(path)
  funcl.sort()
  for f in funcl:
    if showlinenum == "y":
      print(f)
    else:
      print(f[0],f[1])
    
